"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.rollbackEntities = exports.optimisticUpdateEntities = exports.updateEntities = void 0;

var _immutable = require("immutable");

var updateEntities = function updateEntities() {
  var update = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var entities = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : (0, _immutable.Map)();
  var transformed = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : (0, _immutable.Map)();
  // If update is not supplied, then no change to entities will be made
  return Object.keys(update || {}).reduce(function (accum, key) {
    return accum.set(key, update[key]((0, _immutable.fromJS)(entities.get(key)), (0, _immutable.fromJS)(transformed.get(key))));
  }, (0, _immutable.Map)());
};

exports.updateEntities = updateEntities;

var optimisticUpdateEntities = function optimisticUpdateEntities() {
  var optimisticUpdate = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var entities = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : (0, _immutable.Map)();
  return Object.keys(optimisticUpdate).reduce(function (accum, key) {
    if (optimisticUpdate[key]) {
      return accum.set(key, optimisticUpdate[key]((0, _immutable.fromJS)(entities.get(key))));
    }

    return accum.set(key, entities.get(key));
  }, (0, _immutable.Map)());
};

exports.optimisticUpdateEntities = optimisticUpdateEntities;

var rollbackEntities = function rollbackEntities() {
  var rollback = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var initialEntities = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : (0, _immutable.Map)();
  var entities = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : (0, _immutable.Map)();
  return Object.keys(initialEntities.toObject()).reduce(function (accum, key) {
    if (rollback[key]) {
      return accum.set(key, rollback[key](initialEntities.getIn([key]), entities.getIn([key])));
    } else {
      // Default to just reverting to the initial state for that
      // entity (before the optimistic update)
      return accum.set(key, initialEntities.getIn([key]));
    }
  }, (0, _immutable.Map)());
};

exports.rollbackEntities = rollbackEntities;