// @flow

import { Map, fromJS } from 'immutable';
import type { Map as MapType } from 'immutable';
import * as actionTypes from '../constants/action-types';

import type { Action } from '../actions';
import type { QueryKey } from '../types';

export type State = MapType<QueryKey, MapType<string, any>>;

const initialState = Map();

const queries = (state: State = initialState, action: Action): State => {
  switch (action.type) {
    case actionTypes.RESET: {
      return initialState;
    }
    case actionTypes.MUTATE_START:
    case actionTypes.REQUEST_START: {
      const query = Map({
        url: action.url,
        isFinished: false,
        isPending: true,
        isMutation: action.type === actionTypes.MUTATE_START,
        queryCount: state.get(action.queryKey)
          ? state.getIn([action.queryKey, 'queryCount']) + 1
          : 1,
      });

      return state.setIn([action.queryKey], query);
    }
    case actionTypes.REQUEST_SUCCESS:
    case actionTypes.MUTATE_FAILURE:
    case actionTypes.MUTATE_SUCCESS:
    case actionTypes.REQUEST_FAILURE: {
      const query = Map({
        isFinished: true,
        isPending: false,
        lastUpdated: action.time,
        status: action.status,
        headers: fromJS(action.responseHeaders),
      });

      return state.mergeIn([action.queryKey], query);
    }
    case actionTypes.CANCEL_QUERY: {
      if (action.queryKey && state.getIn([action.queryKey, 'isPending'], false)) {
        // Make sure request is actually pending

        const query = Map({
          isFinished: true,
          isPending: false,
          status: 0,
        });

        return state.mergeIn([action.queryKey], query);
      }

      return state;
    }
    default: {
      return state;
    }
  }
};

export default queries;
