// @flow

import type { State as QueriesState } from '../reducers/queries';
import { getQueryKey } from '../lib/query-key';
import type { QueryConfig } from '../types';

export const isFinished = (queriesState: QueriesState, queryConfig: ?QueryConfig): boolean => {
  const queryKey = getQueryKey(queryConfig);

  if (!queryKey) {
    return false;
  }

  return queriesState.getIn([queryKey, 'isFinished'], false);
};

export const isPending = (queriesState: QueriesState, queryConfig: ?QueryConfig): boolean => {
  const queryKey = getQueryKey(queryConfig);

  if (!queryKey) {
    return false;
  }

  return queriesState.getIn([queryKey, 'isPending'], false);
};

export const status = (queriesState: QueriesState, queryConfig: ?QueryConfig): ?number => {
  const queryKey = getQueryKey(queryConfig);

  if (!queryKey) {
    return null;
  }

  return queriesState.getIn([queryKey, 'status'], null);
};

export const headers = (
  queriesState: QueriesState,
  queryConfig: ?QueryConfig,
): ?{ [key: string]: any } => {
  const queryKey = getQueryKey(queryConfig);

  if (!queryKey) {
    return null;
  }

  return queriesState.getIn([queryKey, 'headers'], null);
};

export const lastUpdated = (queriesState: QueriesState, queryConfig: ?QueryConfig): ?number => {
  const queryKey = getQueryKey(queryConfig);

  if (!queryKey) {
    return null;
  }

  return queriesState.getIn([queryKey, 'lastUpdated'], null);
};

export const queryCount = (queriesState: QueriesState, queryConfig: ?QueryConfig): number => {
  const queryKey = getQueryKey(queryConfig);

  if (!queryKey) {
    return 0;
  }

  return queriesState.getIn([queryKey, 'queryCount'], 0);
};
